package com.ywt.glasses.mq;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.ywt.biz.common.constant.KafkaRecordParamConstants;
import com.ywt.biz.common.constant.KafkaTopicsConstants;
import com.ywt.biz.common.util.Checker;
import com.ywt.glasses.api.mg.order.PaymentCallbackRequest;
import com.ywt.glasses.api.mg.order.RefundCallbackRequest;
import com.ywt.glasses.service.MgGoodsOrdersService;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.kafka.annotation.KafkaListener;
import org.springframework.kafka.support.Acknowledgment;
import org.springframework.stereotype.Component;

/**
 * @author Walker
 * Created on 2024/1/15
 */

@Component
@ConditionalOnProperty(value = "spring.kafka.consumer.enable",havingValue = "true")
public class KafkaMsgListener {
    private final Logger logger = LoggerFactory.getLogger(KafkaListener.class);

    public static final String GROUP_NAME = "glasses-front-service";

    @Autowired
    private MgGoodsOrdersService mgGoodsOrdersService;

    @KafkaListener(topics = KafkaTopicsConstants.REFUNDCALLBACK_GLASSESAPISERVICE_REST_MSG, groupId = GROUP_NAME)
    public void refundCallback(ConsumerRecord<String, String> record, Acknowledgment ack) {
        logger.info("KafkaMsgListener#refundCallback(record={}): 开始消费", record);
        try{
            boolean isValid = JSON.isValid(record.value());
            if(isValid){
                JSONObject jsonObject = JSONObject.parseObject(record.value());
                int bizId = Checker.getIntegerValue(jsonObject.getInteger(KafkaRecordParamConstants.GLASSES_REFUND_CALLBACK_BIZ_ID));
                int refundStatus = Checker.getIntegerValue(jsonObject.getInteger(KafkaRecordParamConstants.GLASSES_REFUND_CALLBACK_REFUND_STATUS));
                String refundNo = Checker.getStringValue(jsonObject.getString(KafkaRecordParamConstants.GLASSES_REFUND_CALLBACK_REFUND_NO));
                String custom = Checker.getStringValue(jsonObject.getString(KafkaRecordParamConstants.GLASSES_REFUND_CALLBACK_CUSTOM));
                RefundCallbackRequest request = new RefundCallbackRequest(bizId, refundStatus, refundNo, custom);
                mgGoodsOrdersService.refundCallback(request);
                logger.info("KafkaMsgListener#refundCallback(record={}): 消费完成", record);
            }
        }catch (Exception e){
            logger.error("KafkaMsgListener#refundCallback(record={}):\n {}", record, e.getMessage(), e);
        }
        ack.acknowledge();
    }

    @KafkaListener(topics = KafkaTopicsConstants.PAYMENTCALLBACK_GLASSESAPISERVICE_REST_MSG, groupId = GROUP_NAME)
    public void paymentCallback(ConsumerRecord<String, String> record, Acknowledgment ack) {
        logger.info("KafkaMsgListener#paymentCallback(record={}): 开始消费", record);
        try{
            boolean isValid = JSON.isValid(record.value());
            if(isValid){
                JSONObject jsonObject = JSONObject.parseObject(record.value());
                int bizId = Checker.getIntegerValue(jsonObject.getInteger(KafkaRecordParamConstants.GLASSES_PAYMENT_CALLBACK_BIZ_ID));
                String paymentNo = Checker.getStringValue(jsonObject.getString(KafkaRecordParamConstants.GLASSES_PAYMENT_CALLBACK_PAYMENT_NO));
                int paymentChannel = Checker.getIntegerValue(jsonObject.getInteger(KafkaRecordParamConstants.GLASSES_PAYMENT_CALLBACK_PAYMENT_CHANNEL));
                PaymentCallbackRequest request = new PaymentCallbackRequest(bizId, paymentNo, paymentChannel);
                mgGoodsOrdersService.paymentCallback(request);
                logger.info("KafkaMsgListener#paymentCallback(record={}): 消费完成", record);
            }
        }catch (Exception e){
            logger.error("KafkaMsgListener#paymentCallback(record={}):\n {}", record, e.getMessage(), e);
        }
        ack.acknowledge();
    }
}
